;; emacs-project.el --- project for Emacs C source code  -*- lexical-binding: t -*-

(require 'project)
(require 'uniquify-files)
(require 'file-complete-root-relative)

(cl-defstruct (emacs-project)
  root
  )

(defun emacs-project-try (dir)
  "For 'project-find-functions'."
  (when (string-match "emacs" dir)
    (make-emacs-project :root (locate-dominating-file dir "autogen.sh"))))

(cl-defmethod project-library-roots ((_prj emacs-project))
  nil)

(cl-defmethod project-roots ((prj emacs-project))
  (list (emacs-project-root prj)))

(defun emacs-project-ignore-dir (dir)
  "Return non-nil if 'dir' should be ignored in file-completion-table."
  ;; If DIR is ignored, no children of DIR will be passed to this function.
  (string-equal ".git" (file-name-nondirectory dir)))

(defcustom emacs-project-completion-style 'file-root-rel
  "Completion style for emacs `project-file-completion-table';
determines which completion table to return."
  :group 'minibuffer ;; same as other completion options
  :type '(choice
	  (const default) ;; defers to project-file-completion-styles
	  (const file-root-rel)
	  (const uniquify-file))
  )

(cl-defmethod project-file-completion-table ((prj emacs-project) dirs)
  (let ((iter (make-path-iterator
	       :user-path-non-recursive nil
	       :user-path-recursive (or dirs (list (emacs-project-root prj)))
	       :ignore-function #'emacs-project-ignore-dir)))
    (cl-ecase emacs-project-completion-style
      (default
	(path-iter-all-files iter))

      (file-root-rel
       (apply-partially 'fc-root-rel-completion-table-iter iter))

      (uniquify-file
       (apply-partially #'uniq-file-completion-table iter))
    )))

(add-to-list 'project-find-functions #'emacs-project-try)

(provide 'emacs-project)
;; end of file
